@@ -1,5 +1,4 @@
-#' Simulate response patterns from an underlying multivariate normal
-#' distribution
+#' Simulate response patterns 
 #' 
 #' Simulates response patterns for compensetory and noncompensatory MIRT models
 #' from multivariate normally distributed factor (\eqn{\theta}) scores, or from
@@ -13,6 +12,18 @@
 #' @param d a matrix of intercepts. The matrix should have as many columns as
 #' the item with the largest number of categories, and filled empty locations
 #' with \code{NA}
+#' @param itemtype a character vector of length \code{nrow(a)} specifying the type of items to simulate. 
+#' Can be \code{'dich', 'graded', 'gpcm','nominal'}, or \code{'partcomp'}, for 
+#' dichotomous, graded, generalized 
+#' partial credit, nominal, and partially compensatory models. Note that 
+#' for the gpcm and nominal model there should be as many parameters as desired categories,
+#' however to parameterized them for meaningful interpretation the first category intercept should 
+#' equal 0 for both models
+#' @param nominal a matrix of specific item category slopes for nominal models.
+#' Should be the dimensions as the intecept specification with one less column, with \code{NA}
+#' in locations where not applicable. Note that during estimation the first slope will be constrained
+#' to 0 and the last will be constrained to the number of categories minus 1, 
+#' so it is best to set these as the values for the first and last categories as well
 #' @param N sample size
 #' @param guess a vector of guessing parameters for each item; only applicable
 #' for dichotomous items. Must be either a scalar value that will affect all of
@@ -23,12 +34,6 @@
 #' the identity matrix
 #' @param mu a mean vector of the underlying distribution. Default is a vector
 #' of zeros
-#' @param partcomp a logical vector used to specify which items are to be
-#' treated as partially compensatory items (see Reckase, 2009), and single
-#' values are repeated for each item. Note that when simulating noncompensatory
-#' data the slope parameters must equal the number of intercept parameters
-#' @param factor.loads logical; are the slope parameters in \code{a} in the
-#' factor loadings metric?
 #' @param Theta a user specified matrix of the underlying ability parameters,
 #' where \code{nrow(Theta) == N} and \code{ncol(Theta) == ncol(a)}
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
@@ -79,12 +84,12 @@
 #' 
 #' mu <- c(-.4, -.7, .1)
 #' sigma <- matrix(c(1.21,.297,1.232,.297,.81,.252,1.232,.252,1.96),3,3)
+#' itemtype <- rep('dich', nrow(a))
 #' 
-#' dataset1 <- simdata(a, d, 2000)
-#' dataset2 <- simdata(a, d, 2000, mu = mu, sigma = sigma)
+#' dataset1 <- simdata(a, d, 2000, itemtype)
+#' dataset2 <- simdata(a, d, 2000, itemtype, mu = mu, sigma = sigma)
 #' 
 #' ###An example of a mixed item, bifactor loadings pattern with correlated specific factors
-#' # can use factor loadings metric
 #' a <- matrix(c(
 #' .8,.4,NA,
 #' .4,.4,NA,
@@ -93,40 +98,22 @@
 #' .4,NA,.4,
 #' .7,NA,.4),ncol=3,byrow=TRUE)
 #' 
-#' #first three items are dichotomous, next two have 4 categories, and the last has 3
 #' d <- matrix(c(
 #' -1.0,NA,NA,
 #'  1.5,NA,NA,
 #'  0.0,NA,NA,
-#' 3.0,2.0,-0.5,
-#' 2.5,1.0,-1,
+#' 0.0,-1.0,1.5,  #the first 0 here is the recommended constraint for nominal
+#' 0.0,1.0,-1, #the first 0 here is the recommended constraint for gpcm 
 #' 2.0,0.0,NA),ncol=3,byrow=TRUE)
 #' 
+#' nominal <- matrix(NA, nrow(d), ncol(d))
+#' nominal[4, ] <- c(0,1.2,2) #the first 0 and last (ncat - 1) = 2 values are the recommended constraints
+#' 
 #' sigma <- diag(3)
 #' sigma[2,3] <- sigma[3,2] <- .25
+#' items <- c('dich','dich','dich','nominal','gpcm','graded')
 #' 
-#' dataset <- simdata(a,d,1000,sigma=sigma,factor.loads=TRUE)
-#' 
-#' ####Noncompensatory item example
-#' a <- matrix(c(
-#'   1,NA,
-#' 1.5,NA,
-#'  NA, 1,
-#'  NA,1.6,
-#' 1.5,.5,
-#'  .7, 1), ncol=2,byrow=TRUE)
-#' 
-#' #notice that the partially compensatory items have the same number of intercepts as 
-#' #factors influencing the item
-#' d <- matrix(c(
-#' -1.0,NA,
-#'  1.5,NA,
-#'  0.0,NA,
-#'  3.0,NA,
-#' 2.5,1.0,
-#' 2.0, -1),ncol=2,byrow=TRUE)
-#' 
-#' compdata <- simdata(a,d,3000, partcomp = c(F,F,F,F,T,T))
+#' dataset <- simdata(a,d,1000,items,sigma=sigma,nominal=nominal)
 #'
 #' ####Unidimensional nonlinear factor pattern
 #' theta <- rnorm(2000)
@@ -140,58 +127,45 @@
 #' .4,NA,
 #' .7,NA),ncol=2,byrow=TRUE)
 #' d <- matrix(rnorm(6))
+#' itemtype <- rep('dich',6)
 #' 
-#' nonlindata <- simdata(a,d,2000,Theta=Theta)
+#' nonlindata <- simdata(a,d,2000,itemtype,Theta=Theta)
 #'
 #'    }
 #' 
-simdata <- function(a, d, N, sigma = NULL, mu = NULL, guess = 0, 
-	upper = 1, partcomp = FALSE, factor.loads = FALSE, Theta = NULL)
-{
-	dist = 'normal'
+simdata <- function(a, d, N, itemtype, sigma = NULL, mu = NULL, guess = 0, 
+	upper = 1, nominal = NULL, Theta = NULL)
+{    
 	nfact <- ncol(a)
-	nitems <- nrow(a)	
-	if(factor.loads){
-		cs <- sqrt(1 - rowSums(a^2, na.rm = TRUE))
-		a <- a / cs
-	}
-	K <- rep(0,nitems)
-	for(i in 1:nitems) K[i] <- sum(!is.na(d[i,]))		
-	if(length(partcomp) == 1) partcomp <- rep(partcomp,nitems)
-	if(length(partcomp) != nitems) stop("Logical partcomp vector is incorrect")  
+	nitems <- nrow(a)		
+	K <- rep(0,nitems)	
 	if(length(guess) == 1) guess <- rep(guess,nitems)	
 	if(length(guess) != nitems) stop("Guessing parameter is incorrect")
 	if(length(upper) == 1) upper <- rep(upper,nitems)    
 	if(length(upper) != nitems) stop("Upper bound parameter is incorrect")
-	guess[K > 1] <- 0
-    upper[K > 1] <- 1
+    for(i in 1:length(K)){
+        K[i] <- length(na.omit(d[i, ])) + 1
+        if(any(itemtype[i] == c('gpcm', 'nominal'))) K[i] <- K[i] - 1
+    }
+    guess[K > 2] <- upper[K > 2] <- NA	
 	if(is.null(sigma)) sigma <- diag(nfact)
 	if(is.null(mu)) mu <- rep(0,nfact)
-	if (!is.null(Theta))
+	if(!is.null(Theta))
 		if(ncol(Theta) != nfact || nrow(Theta) != N) 
 			stop("The input Theta matrix does not have the correct dimensions")
-	if (dist == 'normal' && is.null(Theta)) Theta <- rmvnorm(N,mu,sigma)     
-	data <- matrix(0,N,nitems)
-	K[partcomp]	<- 1	
+	if(is.null(Theta)) Theta <- mvtnorm::rmvnorm(N,mu,sigma)     
+    if(is.null(nominal)) nominal <- matrix(NA, nitems, 1)
+	data <- matrix(0, N, nitems)	
+    a[is.na(a)] <- 0    
 	for(i in 1:nitems){
-		if(K[i] == 1){	
-			slp <- a[i,!is.na(a[i,])]
-			tht <- Theta[,!is.na(a[i,])]
-			if(partcomp[i]){										
-				P <- P.comp(slp, na.omit(d[i,]), tht, guess[i], upper[i])				
-			} else {
-				if(length(slp) == 1) tht <- matrix(tht)			
-				P <- P.mirt(slp, na.omit(d[i,1]), tht, guess[i], upper[i])
-			}	
-			for (j in 1:N) data[j,i] <- sample(c(0,1), 1, prob = c((1 - P[j]), P[j]))		
-		} else {			
-			int <- d[i,!is.na(d[i,])]
-			slp <- a[i,!is.na(a[i,])]
-			tht <- Theta[,!is.na(a[i,])]
-			if(length(slp) == 1) tht <- matrix(tht)
-			P <- P.poly(slp, int, tht, itemexp = TRUE)	
-			for (j in 1:N) data[j,i] <- sample(1:ncol(P), 1, prob = P[j,])				  
-		}	  
+        par <- na.omit(c(a[i, ],nominal[i,],d[i,],guess[i],upper[i]))
+        obj <- new(itemtype[i], par=par, nfact=nfact, bfactor=FALSE)
+        if(any(itemtype[i] == c('gpcm','nominal'))) 
+            obj@ncat <- K[i]
+        P <- ProbTrace(obj, Theta)
+		for (j in 1:N) 
+            data[j,i] <- sample(1:ncol(P), 1, prob = P[j,])        
+        if(any(itemtype[i] == c('dich', 'gpcm'))) data[ ,i] <- data[ ,i] - 1 
 	}
 	colnames(data) <- paste("Item_", 1:nitems, sep="") 
 	return(data)
